// -*- C++ -*-
/*!
 * @file InPortPullConnector.h
 * @brief InPortPull type connector class
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2009
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef RTC_OUTPORTPULLCONNECTOR_H
#define RTC_OUTPORTPULLCONNECTOR_H

#include <rtm/InPortConnector.h>
#include <rtm/InPortProvider.h>
#include <rtm/PublisherBase.h>
#include <rtm/DataPortStatus.h>

namespace RTC
{
  class OutPortConsumer;
  class ConnectorListeners;

  /*!
   * @if jp
   * @class InPortPullConnector
   * @brief InPortPullConnector NX
   *
   * InPort  pull ^f[^t[̂߂ Connector NXB̃Iu
   * WFNǵAڑ dataflow_type  pull w肳ꂽꍇA
   * InPort ɂĐELAOutPortPullConnector Ƒ΂ɂȂāA
   * f[^|[g pull ^̃f[^t[B̐ڑɑ΂āA
   * ̃f[^Xg[񋟂B Connector ΉB
   * Connector  ڑɐ UUID ` ID ɂʂB
   *
   * InPortPullConnector ͈ȉ̎O̃IuWFNgLǗB
   *
   * - InPortConsumer
   * - Buffer
   *
   * OutPort ɏ܂ꂽf[^ OutPortPullConnector::write() ɓn
   *  Buffer ɏ܂BInPort::read(),
   * InPortPullConnector::read() ͌ʂƂāAOutPortConsumer::get()
   * ĂяoAOutPortPullConnector ̎obt@f[^ǂݏo
   * AInPortPullConnector ̂obt@Ƀf[^ށB
   *
   * @since 1.0.0
   *
   * @else
   * @class InPortPullConnector
   * @brief InPortPullConnector class
   *
   * Connector class of InPort for pull type dataflow. When "pull" is
   * specified as dataflow_type at the time of establishing
   * connection, this object is generated and owned by the InPort.
   * This connector and InPortPullConnector make a pair and realize
   * pull type dataflow of data ports. One connector corresponds to
   * one connection which provides a data stream. Connector is
   * distinguished by ID of the UUID that is generated at establishing
   * connection.
   *
   * InPortPullConnector owns and manages the following objects.
   *
   * - InPortConsumer
   * - Buffer
   *
   * Data written into the OutPort is passed to the
   * OutPortPullConnector::write(), and is written into the buffer.
   * Consequently, InPort::read() and InPortPullConnector::read() call
   * OutPortConsumer::get(), and it reads data from the buffer of
   * OutPortPullConnector.  Finally data would be written into the
   * InPortPullConnector's buffer.
   *
   * @since 1.0.0
   *
   * @endif
   */
  class InPortPullConnector
    : public InPortConnector
  {
  public:
    DATAPORTSTATUS_ENUM

    /*!
     * @if jp
     * @brief RXgN^
     *
     * InPortPullConnector ̃RXgN^̓IuWFNgɉL
     * ɂƂBConnectorInfo ͐ڑ܂݁Ȁɏ]ob
     * t@𐶐BOutPort C^[tF[X̃voC_IuWFN
     * gւ̃|C^AL̂ŁAInPortPullConnector 
     * OutPortConsumer ̉̐ӔCBeCxgɑ΂R[ob
     * N@\񋟂 ConnectorListeners AK؂ȃ^C~OŃR[
     * obNĂяoBf[^obt@ InPortBase 񋟂
     * ꍇ͂̃|C^B
     *
     * @param info ConnectorInfo
     * @param consumer OutPortConsumer
     * @param listeners ConnectorListeners ^̃XiIuWFNgXg
     * @param buffer CdrBufferBase ^̃obt@
     *
     * @else
     * @brief Constructor
     *
     * InPortPullConnector's constructor is given the following
     * arguments.  According to ConnectorInfo which includes
     * connection information, a buffer is created.  It is also given
     * a pointer to the consumer object for the OutPort interface.
     * The owner-ship of the pointer is owned by this
     * OutPortPullConnector, it has responsibility to destruct the
     * OutPortConsumer.  OutPortPullConnector also has
     * ConnectorListeners to provide event callback mechanisms, and
     * they would be called at the proper timing.  If data buffer is
     * given by OutPortBase, the pointer to the buffer is also given
     * as arguments.
     *
     * @param info ConnectorInfo
     * @param consumer OutPortConsumer
     * @param listeners ConnectorListeners type lsitener object list
     * @param buffer CdrBufferBase type buffer
     *
     * @endif
     */
    InPortPullConnector(ConnectorInfo info,
                        OutPortConsumer* consumer,
                        ConnectorListeners& listeners,
                        CdrBufferBase* buffer = 0);

    /*!
     * @if jp
     * @brief fXgN^
     *
     * disconnect() Ă΂Aconsumer, publisher, buffer ́E폜B
     *
     * @else
     *
     * @brief Destructor
     *
     * This operation calls disconnect(), which destructs and deletes
     * the consumer, the publisher and the buffer.
     *
     * @endif
     */
    virtual ~InPortPullConnector();

    /*!
     * @if jp
     * @brief read ֐
     *
     * OutPortConsumer f[^擾BɓǂݏoꍇA߂
     * l PORT_OK ƂȂAdata ɓǂݏoꂽf[^i[B
     * ȊȌꍇɂ́AG[lƂ BUFFER_EMPTY, TIMEOUT,
     * PRECONDITION_NOT_MET, PORT_ERROR ԂB
     *
     * @return PORT_OK              I
     *         BUFFER_EMPTY         obt@͋ł
     *         TIMEOUT              ^CAEg
     *         PRECONDITION_NOT_MET O𖞂Ȃ
     *         PORT_ERROR           ̑̃G[
     *
     * @else
     * @brief Destructor
     *
     * This function get data from OutPortConsumer.  If data is read
     * properly, this function will return PORT_OK return code. Except
     * normal return, BUFFER_EMPTY, TIMEOUT, PRECONDITION_NOT_MET and
     * PORT_ERROR will be returned as error codes.
     *  
     * @return PORT_OK              Normal return
     *         BUFFER_EMPTY         Buffer empty
     *         TIMEOUT              Timeout
     *         PRECONDITION_NOT_MET Preconditin not met
     *         PORT_ERROR           Other error
     *
     * @endif
     */
    virtual ReturnCode read(cdrMemoryStream& data);

    /*!
     * @if jp
     * @brief ڑ֐
     *
     * Connector ێĂڑ
     *
     * @else
     * @brief Disconnect connection
     *
     * This operation disconnect this connection
     *
     * @endif
     */
    virtual ReturnCode disconnect();

    /*!
     * @if jp
     * @brief ANeBu
     *
     * ̃RlN^ANeBu
     *
     * @else
     *
     * @brief Connector activation
     *
     * This operation activates this connector
     *
     * @endif
     */
    virtual void activate(){}; // do nothing

    /*!
     * @if jp
     * @brief ANeBu
     *
     * ̃RlN^ANeBu
     *
     * @else
     *
     * @brief Connector deactivation
     *
     * This operation deactivates this connector
     *
     * @endif
     */
    virtual void deactivate(){}; // do nothing

  protected:
    /*!
     * @if jp
     * @brief Buffer̐
     *
     * ^ꂽڑɊÂobt@𐶐B
     *
     * @param info ڑ
     * @return obt@ւ̃|C^
     *
     * @else
     * @brief create buffer
     *
     * This function creates a buffer based on given information.
     *
     * @param info Connector information
     * @return The poitner to the buffer
     *
     * @endif
     */
    CdrBufferBase* createBuffer(ConnectorInfo& info);

    /*!
     * @if jp
     * @brief ڑmɃR[obNĂ
     * @else
     * @brief Invoke callback when connection is established
     * @endif
     */
    void onConnect();

    /*!
     * @if jp
     * @brief ڑؒfɃR[obNĂ
     * @else
     * @brief Invoke callback when connection is destroied
     * @endif
     */
    void onDisconnect();
    
  private:
    /*!
     * @if jp
     * @brief OutPortConsumer ւ̃|C^
     * @else
     * @brief the pointer to the OutPortConsumer
     * @endif
     */
    OutPortConsumer* m_consumer;
    
    /*!
     * @if jp
     * @brief ConnectorListenrs ւ̎Q
     * @else
     * @brief A reference to a ConnectorListener
     * @endif
     */
    ConnectorListeners& m_listeners;
  };
}; // namespace RTC

#endif  // RTC_PULL_CONNECTOR_H
